//+------------------------------------------------------------------+
//|                                 UniFisherTransform_v1.2 600+.mq4 |
//|                                Copyright ?2015, TrendLaboratory |
//|            http://finance.groups.yahoo.com/group/TrendLaboratory |
//|                                   E-mail: igorad2003@yahoo.co.uk |
//+------------------------------------------------------------------+
// List of Prices:
// Price    = 0 - Close  
// Price    = 1 - Open  
// Price    = 2 - High  
// Price    = 3 - Low  
// Price    = 4 - Median Price   = (High+Low)/2  
// Price    = 5 - Typical Price  = (High+Low+Close)/3  
// Price    = 6 - Weighted Close = (High+Low+Close*2)/4

 
#property copyright "Copyright ?2015, TrendLaboratory"
#property link      "http://finance.groups.yahoo.com/group/TrendLaboratory"

#property indicator_separate_window
#property indicator_buffers 6
#property indicator_color1 Red
#property indicator_color2 Maroon
#property indicator_color3 LimeGreen
#property indicator_color4 Green
#property indicator_color5 Red
#property indicator_color6 Yellow

#property indicator_width1 5
#property indicator_width2 4
#property indicator_width3 5
#property indicator_width4 4
#property indicator_width5 1
#property indicator_width6 1
#property indicator_style6 2  

//---- indicator parameters

extern int     TimeFrame         =     0;    //TimeFrame in min
extern int     MathMode          =     1;    //Math method: 0-RSI;1-Stoch;2-DMI;
extern int     Price             =     0;    //Price = 0...10 (see List of Prices)
extern int     Length            =    10;    //Period of evaluation
extern int     PreSmooth         =     5;    //Period of PreSmoothing
extern int     Smooth            =     3;    //Period of smoothing
extern int     Signal            =     1;    //Period of Signal Line
extern int     Pole              =     1;    //Pole:1-EMA(1 pole),2-DEMA(2 poles),3-TEMA(3 poles),4-QEMA(4 poles)...
extern int     Order             =     1;    //Smoothing Order(min. 1)
extern double  WeightFactor      =     2;    //Weight Factor (ex.Wilder=1,EMA=2)   
extern double  DampingFactor     =     1;    //Damping Factor

extern string  alerts            = "--- Alerts & Emails ---";
extern int     AlertMode         =     0;
extern int     AlertShift        =     1;    //Alert Shift:0-current bar,1-previous bar
extern int     SoundsNumber      =     5;    //Number of sounds after Signal
extern int     SoundsPause       =     5;    //Pause in sec between sounds 
extern string  UpSound           = "alert.wav";
extern string  DnSound           = "alert2.wav";
extern int     EmailMode         =     0;    //0-on,1-off   
extern int     EmailsNumber      =     1;    //0-on,1-off

//---- indicator buffers
double UpBuffer1[];
double UpBuffer2[];
double DnBuffer1[];
double DnBuffer2[];
double Fisher[];
double fSignal[];

//----
double uniema_tmp[][2], mMA[2], mLo[2];
int    uniema_prevtime[], uniema_size, prevtime;

string TF, IndicatorName, short_name, math_name, price_name;
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
{
   if(TimeFrame <= Period()) TimeFrame = Period();
   TF = tf(TimeFrame);
   if(TF  == "N/A") TimeFrame = Period();
//---- indicator buffers mapping
   IndicatorBuffers(6);
   SetIndexBuffer(0,DnBuffer1); SetIndexStyle(0,DRAW_HISTOGRAM);
   SetIndexBuffer(1,DnBuffer2); SetIndexStyle(1,DRAW_HISTOGRAM);
   SetIndexBuffer(2,UpBuffer1); SetIndexStyle(2,DRAW_HISTOGRAM);
   SetIndexBuffer(3,UpBuffer2); SetIndexStyle(3,DRAW_HISTOGRAM);
   SetIndexBuffer(4,   Fisher); SetIndexStyle(4,DRAW_LINE);
   SetIndexBuffer(5,  fSignal); SetIndexStyle(5,DRAW_LINE); 
   
//---- drawing settings
      
   int draw_begin = Bars - iBars(NULL,TimeFrame)*TimeFrame/Period() + Length + PreSmooth + Smooth + Signal;
//---- indicator name
   switch(Price)
   {
   case 0 : price_name = "Close";   break;
   case 1 : price_name = "Open";    break;
   case 2 : price_name = "High";    break;
   case 3 : price_name = "Low";     break;
   case 4 : price_name = "Median";  break;
   case 5 : price_name = "Typical"; break;
   case 6 : price_name = "WtClose"; break;
   }
   
   
   switch(MathMode)
   {
   case 0 : math_name="RSI"  ; break;
   case 1 : math_name="Stoch"; break;
   case 2 : math_name="DMI"  ; break;
   }
   
   IndicatorName = WindowExpertName(); 
   short_name = IndicatorName + "[" + TF + "]("+ math_name + "," + price_name + "," + Length + "," + PreSmooth + "," + Smooth + "," + Signal + ")";
   IndicatorShortName(short_name);
   
   SetIndexDrawBegin(0,draw_begin);
   SetIndexDrawBegin(1,draw_begin);
   SetIndexDrawBegin(2,draw_begin);
   SetIndexDrawBegin(3,draw_begin);
   SetIndexDrawBegin(4,draw_begin);
   SetIndexDrawBegin(5,draw_begin);
   
   SetIndexLabel(0,"DnTrend Increasing");
   SetIndexLabel(1,"DnTrend Decreasing");
   SetIndexLabel(2,"UpTrend Increasing");
   SetIndexLabel(3,"UpTrend Decreasing");
   SetIndexLabel(4,"FisherTransform");
   SetIndexLabel(5,"Signal Line");
   
//----
   uniema_size = Pole*Order;   
   ArrayResize(uniema_tmp,uniema_size*5);
   ArrayResize(uniema_prevtime,5);


//---- initialization done
   
   return(0);
}
//+------------------------------------------------------------------+
//| UniFisherTransform_v1.2 600+                                     |
//+------------------------------------------------------------------+
int start()
{
   int      limit, i, shift, counted_bars = IndicatorCounted(); 
   
   
   if(counted_bars > 0) limit=Bars-counted_bars-1;
   if(counted_bars < 0) return(0);
   if(counted_bars < 1)
   { 
   limit = Bars - 1;   
      for(i=limit;i>=0;i--) 
      {
      UpBuffer1[i] = EMPTY_VALUE;
      UpBuffer2[i] = EMPTY_VALUE;
      DnBuffer1[i] = EMPTY_VALUE;
      DnBuffer2[i] = EMPTY_VALUE;
      fSignal  [i] = EMPTY_VALUE;
      Fisher   [i] = EMPTY_VALUE;
      }
   SetLevelValue(0,0);
   SetLevelStyle(2,1,Silver);
   }   
  
   if(TimeFrame != Period())
	{
   limit = MathMax(limit,TimeFrame/Period());   
      
      for(shift = 0;shift < limit;shift++) 
      {	
      int y = iBarShift(NULL,TimeFrame,Time[shift]);
      
      
      DnBuffer1[shift] = iCustom(NULL,TimeFrame,IndicatorName,0,MathMode,Price,Length,PreSmooth,Smooth,Signal,Pole,Order,WeightFactor,DampingFactor,
                                 "",AlertMode,AlertShift,SoundsNumber,SoundsPause,UpSound,DnSound,EmailMode,EmailsNumber,0,y);      
      DnBuffer2[shift] = iCustom(NULL,TimeFrame,IndicatorName,0,MathMode,Price,Length,PreSmooth,Smooth,Signal,Pole,Order,WeightFactor,DampingFactor,
                                 "",AlertMode,AlertShift,SoundsNumber,SoundsPause,UpSound,DnSound,EmailMode,EmailsNumber,1,y);      
               
      UpBuffer1[shift] = iCustom(NULL,TimeFrame,IndicatorName,0,MathMode,Price,Length,PreSmooth,Smooth,Signal,Pole,Order,WeightFactor,DampingFactor,
                                 "",AlertMode,AlertShift,SoundsNumber,SoundsPause,UpSound,DnSound,EmailMode,EmailsNumber,2,y);      
      UpBuffer2[shift] = iCustom(NULL,TimeFrame,IndicatorName,0,MathMode,Price,Length,PreSmooth,Smooth,Signal,Pole,Order,WeightFactor,DampingFactor,
                                 "",AlertMode,AlertShift,SoundsNumber,SoundsPause,UpSound,DnSound,EmailMode,EmailsNumber,3,y);      
             
      Fisher[shift]    = iCustom(NULL,TimeFrame,IndicatorName,0,MathMode,Price,Length,PreSmooth,Smooth,Signal,Pole,Order,WeightFactor,DampingFactor,
                                 "",AlertMode,AlertShift,SoundsNumber,SoundsPause,UpSound,DnSound,EmailMode,EmailsNumber,4,y);      
      fSignal[shift]   = iCustom(NULL,TimeFrame,IndicatorName,0,MathMode,Price,Length,PreSmooth,Smooth,Signal,Pole,Order,WeightFactor,DampingFactor,
                                 "",AlertMode,AlertShift,SoundsNumber,SoundsPause,UpSound,DnSound,EmailMode,EmailsNumber,5,y);      
      }  
	
	return(0);
	}
	else _UniFisher(limit);

   return(0);
}

//-----

void _UniFisher(int limit)
{   
   double bulls, bears, Bulls, Bears;
   
   for(int shift=limit;shift>=0;shift--) 
   {
   if(prevtime != Time[shift]){mMA[1] = mMA[0]; mLo[1] = mLo[0]; prevtime = Time[shift];}
   
   if(MathMode != 2) double price = iMA(NULL,0,1,0,0,Price,shift); else price = iMA(NULL,0,1,0,0,2,shift);
       
   mMA[0] = price; 
      
      if(shift < Bars-2)
      {
         if (MathMode == 0)
         {
         bulls = 0.5*(MathAbs(mMA[0] - mMA[1]) + (mMA[0] - mMA[1]));
         bears = 0.5*(MathAbs(mMA[0] - mMA[1]) - (mMA[0] - mMA[1]));
         }
         else  
         if (MathMode == 1)
         {
         double up = High[shift];      
         double dn = Low [shift];
            for(int i=1;i<Length;i++)
            {   
            up = MathMax(up,High[shift+i]);
            dn = MathMin(dn,Low [shift+i]);
            }
         bulls = mMA[0] - dn;
         bears = up - mMA[0];
         }
         else   
         if (MathMode == 2)
         {
         mLo[0] = iMA(NULL,0,1,0,0,3,shift);       
               
         bulls = 0.5*(MathAbs(mMA[0] - mMA[1]) + (mMA[0] - mMA[1]));
         bears = 0.5*(MathAbs(mLo[1] - mLo[0]) + (mLo[1] - mLo[0]));
      
            if (bulls > bears) bears = 0;
            else 
            if (bulls < bears) bulls = 0;
            else
            if (bulls == bears) {bulls = 0; bears = 0;}
         }
         
         
      if(MathMode == 1) int len = 1; else len = Length; 
   
      Bulls = UniXMA(0,bulls,len,Pole,Order,WeightFactor,DampingFactor,shift);       
      Bears = UniXMA(1,bears,len,Pole,Order,WeightFactor,DampingFactor,shift);
            
      if(Bulls + Bears > 0) double RS = (Bulls - Bears)/(Bulls + Bears);
      
      double Value = UniXMA(2,RS,PreSmooth,Pole,Order,WeightFactor,DampingFactor,shift);
      
      if(Value > 0.99) Value = 0.99;
      if(Value <-0.99) Value =-0.99;
      
      Fisher[shift] = UniXMA(3,MathLog((1.0 + Value)/(1.0 - Value)),Smooth,Pole,Order,WeightFactor,DampingFactor,shift);
       
         if(Signal > 0) 
         {
            if(Signal > 1) fSignal[shift] = UniXMA(4,Fisher[shift],Signal,Pole,Order,WeightFactor,DampingFactor,shift);     
            else fSignal[shift] = Fisher[shift+1]; 
         }    
      
      
      UpBuffer1[shift] = EMPTY_VALUE;
      UpBuffer2[shift] = EMPTY_VALUE;
      DnBuffer1[shift] = EMPTY_VALUE;
      DnBuffer2[shift] = EMPTY_VALUE;
   
            
         if(Fisher[shift] > 0)
            {if(Fisher[shift] > Fisher[shift+1]) UpBuffer1[shift] = Fisher[shift]; else  UpBuffer2[shift] = Fisher[shift];}
         else               
            {if(Fisher[shift] < Fisher[shift+1]) DnBuffer1[shift] = Fisher[shift]; else  DnBuffer2[shift] = Fisher[shift];}

      }
   }
   
   if(AlertMode > 0)
   {
   bool uptrend = Fisher[AlertShift] > 0 && Fisher[AlertShift+1] <= 0;                  
   bool dntrend = Fisher[AlertShift] < 0 && Fisher[AlertShift+1] >= 0;
      
      if(Signal > 0) 
      {
      bool upcross = Fisher[AlertShift] > fSignal[AlertShift] && Fisher[AlertShift+1] <= fSignal[AlertShift+1];                  
      bool dncross = Fisher[AlertShift] < fSignal[AlertShift] && Fisher[AlertShift+1] >= fSignal[AlertShift+1];
      }
        
      if((uptrend || dntrend)||(Signal > 0 &&(upcross || dncross))) 
      {
         if(isNewBar(TimeFrame))
         {
         BoxAlert(uptrend," : UniFisher crossed over 0 @ "  + DoubleToStr(Close[AlertShift],Digits));   
         BoxAlert(dntrend," : UniFisher crossed under 0 @ " + DoubleToStr(Close[AlertShift],Digits)); 
         BoxAlert(upcross," : UniFisher crossed over Signal @ "  + DoubleToStr(Close[AlertShift],Digits));   
         BoxAlert(dncross," : UniFisher crossed under Signal @ " + DoubleToStr(Close[AlertShift],Digits)); 
         }
      
      WarningSound(uptrend,SoundsNumber,SoundsPause,UpSound,Time[AlertShift]);
      WarningSound(dntrend,SoundsNumber,SoundsPause,DnSound,Time[AlertShift]);
      WarningSound(upcross,SoundsNumber,SoundsPause,UpSound,Time[AlertShift]);
      WarningSound(dncross,SoundsNumber,SoundsPause,DnSound,Time[AlertShift]);   
         
         if(EmailMode > 0)
         {
         EmailAlert(uptrend,"BUY" ," : UniFisher crossed over 0 @ "  + DoubleToStr(Close[AlertShift],Digits),EmailsNumber); 
         EmailAlert(dntrend,"SELL"," : UniFisher crossed under 0 @ " + DoubleToStr(Close[AlertShift],Digits),EmailsNumber); 
         EmailAlert(upcross,"BUY" ," : UniFisher crossed over Signal @ "  + DoubleToStr(Close[AlertShift],Digits),EmailsNumber); 
         EmailAlert(dncross,"SELL"," : UniFisher crossed under Signal @ " + DoubleToStr(Close[AlertShift],Digits),EmailsNumber); 
         }
      }
   }
}
   
//-----
double UniXMA(int index,double price,int len,int pole,int order,double wf,double df,int bar)
{
   int m = index*uniema_size; 
   double alpha = (wf*order)/(len + wf*order-1); 
         
   if(uniema_prevtime[index] != Time[bar])
   {
   for(int k=0;k<order;k++) 
      for(int j=0;j<pole;j++) uniema_tmp[m+pole*k+j][1] = uniema_tmp[m+pole*k+j][0];
   
   uniema_prevtime[index] = Time[bar];
   }
   
   if(bar >= Bars - 2) 
   {
   for(k=0;k<order;k++) 
      for(j=0;j<pole;j++) uniema_tmp[m+pole*k+j][0] = price; 
   }
   else  
   {
      for(k=0;k<order;k++)
      {
         for(j=0;j<pole;j++)
         {   
         uniema_tmp[m+pole*k+j][0] = (1 - alpha)*uniema_tmp[m+pole*k+j][1] + alpha*price; 
         if(j > 0) price += df*(uniema_tmp[m+pole*k][0] - uniema_tmp[m+pole*k+j][0]); else price = uniema_tmp[m+pole*k+j][0];
         }
      }   
   }
   
   return(price); 
}

string tf(int timeframe)
{
   switch(timeframe)
   {
   case PERIOD_M1:   return("M1");
   case PERIOD_M5:   return("M5");
   case PERIOD_M15:  return("M15");
   case PERIOD_M30:  return("M30");
   case PERIOD_H1:   return("H1");
   case PERIOD_H4:   return("H4");
   case PERIOD_D1:   return("D1");
   case PERIOD_W1:   return("W1");
   case PERIOD_MN1:  return("MN1");
   default:          return("N/A");
   }
}

datetime prevnbtime;

bool isNewBar(int tf)
{
   bool res = false;
   
   if(tf >= 0)
   {
      if(iTime(NULL,tf,0) != prevnbtime)
      {
      res   = true;
      prevnbtime = iTime(NULL,tf,0);
      }   
   }
   else res = true;
   
   return(res);
}

string prevmess;
 
bool BoxAlert(bool cond,string text)   
{      
   string mess = IndicatorName + "("+Symbol()+"," + TF + ")" + text;
   
   if (cond && mess != prevmess)
	{
	Alert (mess);
	prevmess = mess; 
	return(true);
	} 
  
   return(false);  
}

datetime pausetime;

bool Pause(int sec)
{
   if(TimeCurrent() >= pausetime + sec) {pausetime = TimeCurrent(); return(true);}
   
   return(false);
}

datetime warningtime;

void WarningSound(bool cond,int num,int sec,string sound,datetime curtime)
{
   static int i;
   
   if(cond)
   {
   if(curtime != warningtime) i = 0; 
   if(i < num && Pause(sec)) {PlaySound(sound); warningtime = curtime; i++;}       	
   }
}

string prevemail;

bool EmailAlert(bool cond,string text1,string text2,int num)   
{      
   string subj = "New " + text1 +" Signal from " + IndicatorName + "!!!";    
   string mess = IndicatorName + "("+Symbol()+","+TF + ")" + text2;
   
   if (cond && mess != prevemail)
	{
	if(subj != "" && mess != "") for(int i=0;i<num;i++) SendMail(subj, mess);  
	prevemail = mess; 
	return(true);
	} 
  
   return(false);  
}